//自機の速度、加速度
var PLAYER_VX=0.2/2;
var PLAYER_VY=-1.0/2;
var PLAYER_AY=0.1/2;

//障害物の最小スピード、最大スピード
var HURDLE_COUNT=5;
var HURDLE_SPEED_MIN=0.1/2;
var HURDLE_SPEED_MAX=0.3/2;

//ゲーム本体、自機、障害物のオブジェクト
var game;
var player;
var hurdle;

//自機、障害物、消滅のアニメーション
var playerWalkAnim;
var hurdleAnim;
var vanishAnim;

//当たり判定
var playerHit;
var hurdleHit;

//自機の状態
var STATE_ALIVE=0;
var STATE_DEAD=1;

//キャラクターのオブジェクト
function Player(){
	
	//変数objがオブジェクト自身を指すようになる
	var obj = this;
	
	//スプライト初期化
	obj.sprite = game.createSprite();
	
	//アニメ設定
	obj.sprite.initAnim(playerWalkAnim);
	
	//当たり判定の設定
	obj.sprite.initHit(playerHit);
	
	//座標と速度の設定
	obj.sprite.setXY(0, YMAX);
	obj.vy=0;
	
	//ジャンプしているかどうか
	obj.jump=false;
	
	//自機の状態
	obj.state=STATE_ALIVE;
	
	
	//一定時間ごとに呼び出す更新処理
	obj.sprite.onUpdate=function(){
		
		//状態に応じて分岐する
		switch(obj.state){
			case STATE_ALIVE:	obj.alive(); break;
			case STATE_DEAD:	obj.dead(); break;
		}
	}
	
	
	//通常状態の処理
	obj.alive=function(){
		
		//座標と速度を取得する
		var x = obj.sprite.getX();
		var y = obj.sprite.getY();
		var vy = obj.vy;
		
		//カーソルキーで左右に移動
		if(game.isKey(KEY_LEFT)) x-=PLAYER_VX;
		if(game.isKey(KEY_RIGHT)) x+=PLAYER_VX;
		
		//画面からはみ出したら座標修正
		if(x<0) x=0;
		if(x>XMAX) x=XMAX;
		
		//ジャンプしていないときの処理
		if(!obj.jump){
			//スペースキーでジャンプする
			if(game.isKey(KEY_SPACE)){
				vy=PLAYER_VY;
				obj.jump=true;
			}
		
			//アニメ進行させる
			if(x!=obj.sprite.getX()){
				obj.sprite.stepAnim();
			}
		}else
		
		//ジャンプしているときの処理
		{
			//座標と速度を更新する
			y+=vy;
			vy+=PLAYER_AY;
			
			//画面の下端に達したら着地する
			if(y>=YMAX){
				y=YMAX;
				obj.jump=false;
			}
		}
		
		//スプライトの処理を更新
		obj.sprite.setXY(x, y);
		
		//速度のプロパティを更新
		obj.vy=vy;
		
		//障害物に接触したときの処理
		if(game.testHitSprite(obj.sprite)){
			
			//消滅のアニメーションに切り替え
			obj.sprite.initAnim(vanishAnim);
			
			//消滅状態にする
			obj.state=STATE_DEAD;
		}
	}
	
	obj.dead=function() {
		obj.sprite.stepAnim();
		if (obj.sprite.isAnimOver()) {
			init();
		}
	}
}

function Hurdle() {
	var obj=this;
	
	//スプライトの作成
	obj.sprite=game.createSprite();
	
	//アニメーションの設定
	obj.sprite.initAnim(hurdleAnim);
	
	//当たり判定設定
	obj.sprite.initHit(hurdleHit);
	
	//座標と速度をランダムに決める
	obj.sprite.setXY(randFloat2(XMAX+1, XMAX*2), YMAX);
	obj.vx=-randFloat2(HURDLE_SPEED_MIN, HURDLE_SPEED_MAX);

	//ゲームの更新処理
	obj.sprite.onUpdate=function() {
		
		//座標と速度を取得
		var x=obj.sprite.getX(), y=obj.sprite.getY();
		var vx=obj.vx;

		//座標の更新
		x+=obj.vx;
		
		//画面の左端にきたら右端に戻し
		//速度をランダムに決めなおす
		if (x<=-1) {
			x=XMAX+1;
			vx=-randFloat2(HURDLE_SPEED_MIN, HURDLE_SPEED_MAX);
		}

		//スプライト座標を更新する
		obj.sprite.setXY(x, y);
		
		//速度のプロパティを更新
		obj.vx=vx;
		
		//転がるアニメーションの再生
		obj.sprite.stepAnim();
	}
}

function init() {
	var i;
	
	//スプライト全消去
	game.clear();
	
	//自機作成
	player=new Player();
	
	//障害物の作成
	hurdle=new Array();
	for (i=0; i<HURDLE_COUNT; i++) {
		hurdle[i]=new Hurdle();
	}
}

function main(){
	
	var i;
	
	//ゲーム本体のオブジェクト初期化
	game = new Game();
	
	//プレイヤーアニメ配列初期化
	playerWalkAnim = new Array(
		"heart00.png","heart00.png","heart00.png",
		"heart01.png","heart01.png","heart01.png",
		"heart02.png","heart02.png","heart02.png",
		"heart03.png","heart03.png","heart03.png"
	);
	
	//敵配列初期化
	hurdleAnim = new Array(
		"saki00.png","saki00.png","saki00.png",
		"saki01.png","saki01.png","saki01.png",
		"saki02.png","saki02.png","saki02.png",
		"saki03.png","saki03.png","saki03.png"
	);
	
	//消滅アニメ
	vanishAnim=new Array(
		"effect01.png","effect01.png",
		"effect02.png","effect02.png",
		"effect03.png","effect03.png"
	);
	
	//当たり判定
	playerHit=new Array(0.23, 0.25, 0.73, 0.75);
	hurdleHit=new Array(0.23, 0.25, 0.73, 0.75);
	
	//ゲームを開始する
	init();
}
	